package com.xf1024.common.service;

import com.xf1024.common.base.model.PagingModel;
import com.xf1024.common.dao.BaseDao;
import com.xf1024.common.dao.MyBatisCustomDao;
import com.xf1024.common.dao.MyBatisGeneratorDao;
import com.xf1024.common.dao.entity.BaseEntity;
import com.xf1024.common.dao.entity.MyBatisBaseExample;
import com.xf1024.common.model.CriteriaModel;
import com.xf1024.common.model.DateBetweenModel;
import com.xf1024.common.model.SortModel;
import com.xf1024.common.util.ExampleUtil;

import java.util.List;

public abstract class DaoSupportService<T> extends BaseSupportService {

    /**
     * 需要重写方法
     * @return
     */
    protected abstract BaseDao getBaseDao();

    protected int insert(T record) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.insert(record);
    }

    protected int insertSelective(T record) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.insertSelective(record);
    }

    protected int updateByPrimaryKeySelective(T record) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByPrimaryKeySelective(record);
    }

    protected int updateByPrimaryKey(T record) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByPrimaryKey(record);
    }

    protected int updateByPrimaryKeyWithBLOBs(T record) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByPrimaryKeyWithBLOBs(record);
    }

    protected int updateByExample(Object record, MyBatisBaseExample example) {
        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByExample(record, example);
    }

    protected int updateByExampleWithBLOBs(Object record, MyBatisBaseExample example) {
        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByExampleWithBLOBs(record, example);
    }

    protected int updateByExampleSelective(T record, MyBatisBaseExample example) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.updateByExampleSelective(record, example);
    }

    protected int deleteByPrimaryKey(Object id) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.deleteByPrimaryKey(id);
    }

    protected int deleteByExample(MyBatisBaseExample example) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.deleteByExample(example);
    }

    protected T selectByPrimaryKey(Object id) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return (T) dao.selectByPrimaryKey(id);
    }

    protected List<T> selectByExample(MyBatisBaseExample example) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return (List<T>) dao.selectByExample(example);
    }

    protected List<T> selectByExampleWithBLOBs(MyBatisBaseExample example) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return (List<T>) dao.selectByExampleWithBLOBs(example);
    }

    protected T selectOneByExampleWithBLOBs(MyBatisBaseExample example) {

        if (null == example.getPageNo()) {
            example.setPageNo(1);
        }
        example.setPageSize(1);
        List<T> list = selectByExampleWithBLOBs(example);
        if (null != list && !list.isEmpty()) {
            return list.get(0);
        }

        return null;
    }

    protected T selectOneByExample(MyBatisBaseExample example) {

        if (null == example.getPageNo()) {
            example.setPageNo(1);
        }
        example.setPageSize(1);
        List<T> list = selectByExample(example);
        if (null != list && !list.isEmpty()) {
            return list.get(0);
        }

        return null;
    }

    protected long countByExample(MyBatisBaseExample example) {

        MyBatisGeneratorDao dao = (MyBatisGeneratorDao) getBaseDao();
        return dao.countByExample(example);
    }

    protected List<T> selectAllByEntity(BaseEntity entity) {

        return selectByEntity(entity, null, null, null);
    }

    protected List<T> selectByEntity(BaseEntity entity, PagingModel pagingModel) {

        return selectByEntity(entity, null, null, pagingModel);
    }

    protected List<T> selectAllByEntity(BaseEntity entity, SortModel sortModel) {

        return selectByEntity(entity, null, sortModel, null);
    }

    protected List<T> selectByEntity(BaseEntity entity, SortModel sortModel, PagingModel pagingModel) {

        return selectByEntity(entity, null, sortModel, pagingModel);
    }

    protected List<T> selectByEntity(BaseEntity entity, DateBetweenModel dateBetweenModel, SortModel sortModel, PagingModel pagingModel) {

        CriteriaModel criteriaModel = new CriteriaModel();
        criteriaModel.setAndEntity(entity);
        criteriaModel.setAndDateBetween(dateBetweenModel);
        MyBatisCustomDao dao = (MyBatisCustomDao) getBaseDao();
        return (List<T>) dao.selectByEntity(criteriaModel, sortModel, pagingModel);
    }


    protected long countByEntity(BaseEntity entity) {

        CriteriaModel criteriaModel = new CriteriaModel();
        criteriaModel.setAndEntity(entity);
        MyBatisCustomDao dao = (MyBatisCustomDao) getBaseDao();
        return dao.countByEntity(criteriaModel);
    }

    protected PagingModel selectByExampleForPaging(MyBatisBaseExample example) {

        PagingModel pagingModel = new PagingModel();
        ExampleUtil.copyPagingInfo(pagingModel, example);
        List<T> list = selectByExample(example);
        pagingModel.setData(list);
        if (null != list && !list.isEmpty()) {
            long count = countByExample(example);
            pagingModel.setDataCount(count);
        }

        return pagingModel;
    }

    protected PagingModel selectByEntityForPaging(BaseEntity entity, DateBetweenModel dateBetweenModel, SortModel sortModel, PagingModel pagingModel) {

        List<T> list = selectByEntity(entity, dateBetweenModel, sortModel, pagingModel);
        pagingModel.setData(list);
        if (null != list && !list.isEmpty()) {
            long count = countByEntity(entity);
            pagingModel.setDataCount(count);
        }

        return pagingModel;
    }

    protected PagingModel selectByEntityForPaging(BaseEntity entity, SortModel sortModel, PagingModel pagingModel) {

        List<T> list = selectByEntity(entity, null, sortModel, pagingModel);
        pagingModel.setData(list);
        if (null != list && !list.isEmpty()) {
            long count = countByEntity(entity);
            pagingModel.setDataCount(count);
        }

        return pagingModel;
    }

    protected PagingModel selectByEntityForPaging(BaseEntity entity, PagingModel pagingModel) {

        return selectByEntityForPaging(entity, null, null, pagingModel);
    }

    protected T selectOneByEntity(BaseEntity entity) {

        PagingModel pagingModel = new PagingModel();
        pagingModel.setPageNo(1);
        pagingModel.setPageSize(1);
        List<T> list = selectByEntity(entity, null, null, pagingModel);
        if (null != list && !list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    protected int deleteByEntity(BaseEntity entity) {

        CriteriaModel criteriaModel = new CriteriaModel();
        criteriaModel.setAndEntity(entity);

        MyBatisCustomDao dao = (MyBatisCustomDao) getBaseDao();
        return dao.deleteByEntity(criteriaModel);
    }

}
