package com.xf1024.common.util.generator;

import com.xf1024.common.dao.MyBatisGeneratorDao;
import com.xf1024.common.dao.entity.MyBatisBaseEntity;
import com.xf1024.common.dao.entity.MyBatisBaseExample;
import org.apache.ibatis.type.JdbcType;
import org.mybatis.generator.api.GeneratedXmlFile;
import org.mybatis.generator.api.IntrospectedColumn;
import org.mybatis.generator.api.IntrospectedTable;
import org.mybatis.generator.api.PluginAdapter;
import org.mybatis.generator.api.dom.java.FullyQualifiedJavaType;
import org.mybatis.generator.api.dom.java.Interface;
import org.mybatis.generator.api.dom.java.Method;
import org.mybatis.generator.api.dom.java.TopLevelClass;
import org.mybatis.generator.api.dom.xml.Attribute;
import org.mybatis.generator.api.dom.xml.Document;
import org.mybatis.generator.api.dom.xml.TextElement;
import org.mybatis.generator.api.dom.xml.XmlElement;
import org.mybatis.generator.codegen.mybatis3.MyBatis3FormattingUtilities;

import java.lang.reflect.Field;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * 
 * @author wangzhengzhen
 *
 */
public class MyBatisGeneratorExtendPlugin extends PluginAdapter {

    @Override
    public boolean sqlMapGenerated(GeneratedXmlFile sqlMap, IntrospectedTable introspectedTable) {
        try {
            // 生成时覆盖原文件
            Field field = sqlMap.getClass().getDeclaredField("isMergeable");
            field.setAccessible(true);
            field.setBoolean(sqlMap, false);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

	@Override
	public boolean clientSelectByExampleWithoutBLOBsMethodGenerated(Method method, Interface interfaze,
			IntrospectedTable introspectedTable) {

		interfaze.addImportedType((new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getName())));
		interfaze.addSuperInterface(new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getSimpleName()));

		return true;
	}

	@Override
	public boolean clientSelectByPrimaryKeyMethodGenerated(Method method, Interface interfaze,
			IntrospectedTable introspectedTable) {
		
		interfaze.addImportedType((new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getName())));
		interfaze.addSuperInterface(new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getSimpleName()));
		
		return true;
	}

	@Override
	public boolean clientSelectByPrimaryKeyMethodGenerated(Method method, TopLevelClass topLevelClass,
			IntrospectedTable introspectedTable) {
		
		topLevelClass.addImportedType((new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getName())));
		topLevelClass.addSuperInterface(new FullyQualifiedJavaType(MyBatisGeneratorDao.class.getSimpleName()));
		
		return true;
	}

	@Override
	public boolean sqlMapDocumentGenerated(Document document, IntrospectedTable introspectedTable) {

        addEntityWhereClauseGenerated(document, introspectedTable);

		return super.sqlMapDocumentGenerated(document, introspectedTable);
	}

    @Override
    public boolean sqlMapCountByExampleElementGenerated(XmlElement element, IntrospectedTable introspectedTable) {

        return super.sqlMapCountByExampleElementGenerated(element, introspectedTable);
    }

    @Override
    public boolean sqlMapSelectByExampleWithBLOBsElementGenerated(XmlElement element, IntrospectedTable introspectedTable) {

        addPagingElementGenerated(element);

        return super.sqlMapSelectByExampleWithBLOBsElementGenerated(element, introspectedTable);
    }

    @Override
    public boolean sqlMapSelectByExampleWithoutBLOBsElementGenerated(XmlElement element, IntrospectedTable introspectedTable) {

        addPagingElementGenerated(element);

        return super.sqlMapSelectByExampleWithoutBLOBsElementGenerated(element, introspectedTable);
    }

    @Override
    public boolean modelBaseRecordClassGenerated(TopLevelClass topLevelClass, IntrospectedTable introspectedTable) {

        topLevelClass.addImportedType((new FullyQualifiedJavaType(MyBatisBaseEntity.class.getName())));
        topLevelClass.setSuperClass(new FullyQualifiedJavaType(MyBatisBaseEntity.class.getName()));
        return super.modelBaseRecordClassGenerated(topLevelClass, introspectedTable);
    }

    @Override
    public boolean modelRecordWithBLOBsClassGenerated(TopLevelClass topLevelClass, IntrospectedTable introspectedTable) {

        topLevelClass.addImportedType((new FullyQualifiedJavaType(MyBatisBaseEntity.class.getName())));
        topLevelClass.setSuperClass(new FullyQualifiedJavaType(MyBatisBaseEntity.class.getName()));
        return super.modelRecordWithBLOBsClassGenerated(topLevelClass, introspectedTable);
    }

    @Override
    public boolean modelExampleClassGenerated(TopLevelClass topLevelClass, IntrospectedTable introspectedTable) {
        topLevelClass.addImportedType((new FullyQualifiedJavaType(MyBatisBaseExample.class.getName())));
        topLevelClass.setSuperClass(new FullyQualifiedJavaType(MyBatisBaseExample.class.getName()));
        return super.modelExampleClassGenerated(topLevelClass, introspectedTable);
    }

    @Override
	public boolean validate(List<String> arg0) {

		return true;
	}

    /**
     * 添加分页支持
     * @param element
     */
    public void addPagingElementGenerated(XmlElement element) {

        XmlElement paging = new XmlElement("if");
        paging.addAttribute(new Attribute("test", "offset!=null and pageSize!=null"));
        paging.addElement(new TextElement(" limit #{offset}, #{pageSize}"));
        element.addElement(paging);
    }

    /**
     * 添加条件
     * @param document
     * @param introspectedTable
     */
    public void addEntityWhereClauseGenerated(Document document, IntrospectedTable introspectedTable) {

        String tableName = introspectedTable.getAliasedFullyQualifiedTableNameAtRuntime();// 数据库表名

//		String parameterType = null;
//		if (introspectedTable.getRules().generatePrimaryKeyClass()) {
//			parameterType = introspectedTable.getPrimaryKeyType();
//		} else {
//         select by primary key, but no primary key class. Fields
//         must be in the base record
//			parameterType = introspectedTable.getBaseRecordType();
//		}

        ///////////// 生成条件 ///////////////
        XmlElement trimElement = new XmlElement("trim");
        trimElement.addAttribute(new Attribute("prefix", " where 1=1 "));

        // and条件
        XmlElement andCriteria = new XmlElement("if");
        andCriteria.addAttribute(new Attribute("test", "criteria.andEntity!=null"));

        // and like条件
        XmlElement andLikeCriteria = new XmlElement("if");
        andLikeCriteria.addAttribute(new Attribute("test", "criteria.andLikeEntity!=null"));

        // or条件
        XmlElement orCriteria = new XmlElement("if");
        orCriteria.addAttribute(new Attribute("test", "criteria.orEntity!=null"));

        // or like条件
        XmlElement orLikeCriteria = new XmlElement("if");
        orLikeCriteria.addAttribute(new Attribute("test", "criteria.orLikeEntity!=null"));

        List<IntrospectedColumn> columnList = new ArrayList<>();
        columnList.addAll(introspectedTable.getPrimaryKeyColumns());
        columnList.addAll(introspectedTable.getBaseColumns());
//        columnList.addAll(introspectedTable.getBLOBColumns()); // 不作条件

        for (IntrospectedColumn introspectedColumn : columnList) {

            if (introspectedColumn.getJdbcType() == JdbcType.LONGNVARCHAR.TYPE_CODE
                    || introspectedColumn.getJdbcType() == JdbcType.LONGVARCHAR.TYPE_CODE
                    || introspectedColumn.getJdbcType() == JdbcType.BINARY.TYPE_CODE
                    || introspectedColumn.getJdbcType() == JdbcType.BLOB.TYPE_CODE) {

                System.out.println("跳过表[" + introspectedTable.getFullyQualifiedTableNameAtRuntime() + "]字段[" + introspectedColumn.getJavaProperty() + "]做为条件");
                continue;
            }

            andCriteria.addElement(getDynamicEqualsCriteria("and", introspectedColumn));
            orCriteria.addElement(getDynamicEqualsCriteria("or", introspectedColumn));
            if (introspectedColumn.getJdbcType() == JdbcType.VARCHAR.TYPE_CODE
                    || introspectedColumn.getJdbcType() == JdbcType.NVARCHAR.TYPE_CODE) {
                andLikeCriteria.addElement(getDynamicLikeCriteria("and", introspectedColumn));
                orLikeCriteria.addElement(getDynamicLikeCriteria("or", introspectedColumn));
            }
        }
        trimElement.addElement(andCriteria);
        trimElement.addElement(andLikeCriteria);
        trimElement.addElement(orCriteria);
        trimElement.addElement(orLikeCriteria);

        XmlElement dateBetween = new XmlElement("if");
        dateBetween.addAttribute(new Attribute("test", "criteria.andDateBetween!=null"));
        XmlElement startDateIsNotNullElement = new XmlElement("if");
        startDateIsNotNullElement.addAttribute(new Attribute("test", "criteria.andDateBetween.startDate!=null"));
        startDateIsNotNullElement.addElement(new TextElement(" and ${criteria.andDateBetween.column}&gt;=#{criteria.andDateBetween.startDate}"));
        dateBetween.addElement(startDateIsNotNullElement);
        XmlElement endDateIsNotNullElement = new XmlElement("if");
        endDateIsNotNullElement.addAttribute(new Attribute("test", "criteria.andDateBetween.endDate!=null"));
        endDateIsNotNullElement.addElement(new TextElement(" and ${criteria.andDateBetween.column}&lt;=#{criteria.andDateBetween.endDate}"));
        dateBetween.addElement(endDateIsNotNullElement);
        trimElement.addElement(dateBetween);

//        XmlElement where = new XmlElement("if");
//        where.addAttribute(new Attribute("test", "criteria.where!=null and criteria.where!=''"));
//        where.addElement(new TextElement("${criteria.where}"));
//        trimElement.addElement(where);

        XmlElement criteria = new XmlElement("if");
        criteria.addAttribute(new Attribute("test", "criteria!=null"));
        criteria.addElement(trimElement);

        XmlElement criteriaElement = new XmlElement("sql");
        criteriaElement.addAttribute(new Attribute("id", "Entity_Where_Clause"));
        criteriaElement.addElement(criteria);

        context.getCommentGenerator().addComment(criteriaElement);

        //////// 生成查询 ///////////////
        XmlElement selectElement = new XmlElement("select");
        selectElement.addAttribute(new Attribute("id", "selectByEntity"));

        if (introspectedTable.getRules().generateResultMapWithBLOBs()) {
            selectElement.addAttribute(new Attribute("resultMap", //$NON-NLS-1$
                    introspectedTable.getResultMapWithBLOBsId()));
        } else {
            selectElement.addAttribute(new Attribute("resultMap", //$NON-NLS-1$
                    introspectedTable.getBaseResultMapId()));
        }
//        if (stringHasValue(introspectedTable.getSelectByPrimaryKeyQueryId())) {
//
//        }
        selectElement.addElement(new TextElement("select "));
        selectElement.addElement(getBaseColumnListElement(introspectedTable));

        if (introspectedTable.hasBLOBColumns()) {
            XmlElement withBlobColumn = new XmlElement("if");
            withBlobColumn.addAttribute(new Attribute("test", "criteria!=null and criteria.resultWithBlobs==true"));
            withBlobColumn.addElement(new TextElement(","));
            withBlobColumn.addElement(getBlobColumnListElement(introspectedTable));
            selectElement.addElement(withBlobColumn);
        }

        selectElement.addElement(new TextElement(" from " + tableName));
        selectElement.addElement(getQueryCriteriaElement(introspectedTable));

        XmlElement orderBy = new XmlElement("if");
        orderBy.addAttribute(new Attribute("test", "sort!=null and sort.column!=null"));
        orderBy.addElement(new TextElement(" order by ${sort.column} ${sort.method}"));
        selectElement.addElement(orderBy);

        XmlElement paging = new XmlElement("if");
        paging.addAttribute(new Attribute("test", "paging!=null"));
        paging.addElement(new TextElement(" limit #{paging.offset}, #{paging.pageSize}"));
        selectElement.addElement(paging);

        context.getCommentGenerator().addComment(selectElement);

        //////////// 查询计数 //////////////
        XmlElement countLogicElement = new XmlElement("select");
        countLogicElement.addAttribute(new Attribute("id", "countByEntity"));
        countLogicElement.addAttribute(new Attribute("resultType","java.lang.Long"));
        countLogicElement.addElement(new TextElement("select count(*) from " + tableName));
        countLogicElement.addElement(getQueryCriteriaElement(introspectedTable));

        context.getCommentGenerator().addComment(countLogicElement);

        /////////// 删除 ////////////////
        XmlElement deleteElement = new XmlElement("delete");
        deleteElement.addAttribute(new Attribute("id", "deleteByEntity"));
        deleteElement.addElement(new TextElement("delete from " + tableName));
        deleteElement.addElement(getQueryCriteriaElement(introspectedTable));

        context.getCommentGenerator().addComment(deleteElement);

        if (context.getPlugins().sqlMapSelectByPrimaryKeyElementGenerated(selectElement, introspectedTable)) {
            XmlElement parentElement = document.getRootElement();
            parentElement.addElement(criteriaElement);
            parentElement.addElement(selectElement);
            parentElement.addElement(countLogicElement);
            parentElement.addElement(deleteElement);
        }
    }

    protected XmlElement getQueryCriteriaElement(IntrospectedTable introspectedTable) {
        XmlElement element = new XmlElement("include");
        element.addAttribute(new Attribute("refid", introspectedTable.getMyBatis3SqlMapNamespace() + ".Entity_Where_Clause"));

        return element;
    }

    protected XmlElement getBaseColumnListElement(IntrospectedTable introspectedTable) {
        XmlElement element = new XmlElement("include");
        element.addAttribute(new Attribute("refid",
                introspectedTable.getMyBatis3SqlMapNamespace()
                        + "." + introspectedTable.getBaseColumnListId()));
        return element;
    }

    protected XmlElement getBlobColumnListElement(IntrospectedTable introspectedTable) {
        XmlElement element = new XmlElement("include");
        element.addAttribute(new Attribute("refid",
                introspectedTable.getMyBatis3SqlMapNamespace()
                        + "." + introspectedTable.getBlobColumnListId()));
        return element;
    }

    private XmlElement getDynamicEqualsCriteria(String operator, IntrospectedColumn introspectedColumn) {

        String javaPropertyName = introspectedColumn.getJavaProperty();
        String pattern = null;
        if (introspectedColumn.getFullyQualifiedJavaType().equals(FullyQualifiedJavaType.getStringInstance())) {
            pattern = "criteria.{0}Entity.{1}!=null and criteria.{0}Entity.{1}!=''''";
        } else {
            pattern = "criteria.{0}Entity.{1}!=null";
        }
        XmlElement element = new XmlElement("if");
        element.addAttribute(new Attribute("test", MessageFormat.format(pattern, operator, javaPropertyName)));
        element.addElement(new TextElement(operator + " " + MyBatis3FormattingUtilities.getAliasedEscapedColumnName(introspectedColumn)
                + " = " + MyBatis3FormattingUtilities.getParameterClause(introspectedColumn, "criteria." + operator + "Entity.")));
        return element;
    }

    private XmlElement getDynamicLikeCriteria(String operator, IntrospectedColumn introspectedColumn) {

        if (introspectedColumn.getJdbcType() == JdbcType.VARCHAR.TYPE_CODE
                || introspectedColumn.getJdbcType() == JdbcType.NVARCHAR.TYPE_CODE) {

            String javaPropertyName = introspectedColumn.getJavaProperty();
            String pattern = "criteria.{0}Entity.{1}!=null and criteria.{0}Entity.{1}!=''''";
            XmlElement element = new XmlElement("if");
            element.addAttribute(new Attribute("test", MessageFormat.format(pattern, operator, javaPropertyName)));
            element.addElement(new TextElement(operator + " " + MyBatis3FormattingUtilities.getAliasedEscapedColumnName(introspectedColumn)
                    + " like '%${criteria." + operator + "LikeEntity." + javaPropertyName + "}%'"));
            return element;
        }

        return null;
    }

}
